******************************************************************************************************;
* Program: ChristmasTip2016.sas                                                                      *;
* Allows the user to load names present in sashelp.class                                             *;
* Created 2016-12-06 by Jonas Wetterberg, SAS Institutes Sweden                                      *;
* E-mail: jonas.wetterberg@sas.com                                                                   *;
******************************************************************************************************;

%stpbegin;

libname indata xml xmlfileref=excel_s Encoding=UTF8;

/* Read the excel data to a normal SAS dataset. First row in Excel will be column name in SAS dataset */

data input_data;
	set indata.excel_table;
run;

%global validation_error;
%let validation_error = FALSE;
data valid_names(drop=rc) 
	 not_valid_names (drop=rc);
	set input_data;

	if _n_ = 1 then do;
		declare hash h1(dataset: 'sashelp.class');
		h1.defineKey('name');
		h1.defineDone();
	end;

	*************************************************************************************************;
	* Note: input data must have a column called name for this step to work. If there is no such    *;
	* column there will be an execution error in this step.                                         *;
	*************************************************************************************************;
	rc = h1.find(); 

	*************************************************************************************************;
	* If rc = 0 then the name was present in sashelp.class, if rc ne 0 there was no match           *;
	*************************************************************************************************;
	if rc = 0 then do;
		output valid_names;
	end;
	else do;
		error_on_row = _n_;
		error_message = catx(' ', name, 'is not a valid name. Please use only names in sashelp.class');
		output not_valid_names;
		* Set macro variable validation_error to true if this happens;
		call symputx('validation_error', 'TRUE', 'G');		
	end;
run;

%macro run_data_load;
	%if &validation_error = FALSE %then %do;
		* call macro or include program that load the data;
	%end;
	%else %do;
		* There was some problem validating the data - inform the user;
		title1 'Data validation errors - errors on the following rows';
		proc print data=not_valid_names label noobs;
		run;
	%end;
%mend;
%run_data_load;

%stpend;

